<?php
/**
 * @package Full Site Cache for CloudFront
 * @version 0.0.2
 */
/*
Plugin Name: Full Site Cache for CloudFront
Description: This plugin allows can auto invalidate the cache automatically if you are using full site CDN.
Author: ZE3kr
Version: 0.0.2
Author URI: https://ze3kr.com/
Text Domain: full-site-cache-cf
Domain Path: /languages
*/

if ( ! defined( 'ABSPATH' ) ) exit;

/**
* Invalidate the whole blog, include all media.
*
* @since   0.0.1
* @change  0.0.1
*/
function tlo_cf_purge_blog() {
	global $tlo_cf_id;

	$distribution = $tlo_cf_id[get_current_blog_id()];
	$epoch = date('U');

	$xml = <<<EOD
<InvalidationBatch>
	<Path>/*</Path>
	<CallerReference>{$distribution}{$epoch}</CallerReference>
</InvalidationBatch>
EOD;
	return tlo_cf_purge($xml);
}

function tlo_cf_purge_blog_cron() {
	$time = wp_cache_get('tlo_cf_last_'.get_current_blog_id());
	if( time() - $time < 0 ){
		return;
	} elseif( time() - $time < 600 ) {
		wp_schedule_single_event($time + 600, 'tlo_cf_purge_blog_hook');
		wp_cache_set('tlo_cf_last_'.get_current_blog_id(), $time + 600, '', 3600);
	} else {
		wp_schedule_single_event(time(), 'tlo_cf_purge_blog_hook');
		wp_cache_set('tlo_cf_last_'.get_current_blog_id(), time(), '', 3600);
	}
}
add_action('tlo_cf_purge_blog_hook', 'tlo_cf_purge_blog');

/**
* Invalidate the post by post path
*
* @since   0.0.1
* @change  0.0.1
*
* @param   array  $path  The path to invalidate
*/
function tlo_cf_purge_post_id($path) {
	global $tlo_cf_id;

	// Purge a post
	$distribution = $tlo_cf_id[get_current_blog_id()];
	$epoch = date('U');
	// Because cache clould have query, so it append a wildcard.
	$xml = <<<EOD
<InvalidationBatch>
	<Path>{$path}*</Path>
	<CallerReference>{$distribution}{$epoch}</CallerReference>
</InvalidationBatch>
EOD;
	tlo_cf_purge($xml);
}

function tlo_cf_purge_post_id_cron($post_ID) {
	$permalink = get_permalink( $post_ID );
	$path = parse_url( $permalink, PHP_URL_PATH );

	$time = wp_cache_get('tlo_cf_last_'.base64_encode($path).get_current_blog_id());
	if( time() - $time < 0 ){
		return;
	} elseif( time() - $time < 600 ) {
		wp_schedule_single_event($time + 600, 'tlo_cf_purge_post_hook', [$path]);
		wp_cache_set('tlo_cf_last_'.base64_encode($path).get_current_blog_id(), $time + 600, '', 3600);
	} else {
		wp_schedule_single_event(time(), 'tlo_cf_purge_post_hook', [$path]);
		wp_cache_set('tlo_cf_last_'.base64_encode($path).get_current_blog_id(), time(), '', 3600);
	}
}
add_action('tlo_cf_purge_post_hook', 'tlo_cf_purge_post_id');

function tlo_cf_purge_comment_status($after_status, $before_status, $comment) {
	if ( $after_status != $before_status ) {
		tlo_cf_purge_post_id_cron( $comment->comment_post_ID );
	}
}

function tlo_cf_purge_comment_edit($id) {
	tlo_cf_purge_post_id_cron( get_comment($id)->comment_post_ID );
}

function tlo_cf_purge_comment_new($approved, $comment) {
	if ( $approved === 1 ) {
		tlo_cf_purge_post_id_cron( $comment['comment_post_ID'] );
	}

	return $approved;
}

function tlo_cf_register_publish_hooks(){
	// get post types
	$post_types = get_post_types(
		['public' => true]
	);

	// check if empty
	if ( empty($post_types) ) {
		return;
	}

	// post type actions
	foreach ( $post_types as $post_type ) {
		add_action( 'publish_' .$post_type, 'tlo_cf_purge_post_id_cron');
		add_action( 'publish_future_' .$post_type, 'tlo_cf_purge_post_id_cron');
	}
}

function on_all_status_transitions( $new_status, $old_status, $post ) {
	if ( ($new_status == 'publish' && $old_status != 'publish') || ($new_status == 'new') || ($old_status == 'publish'  &&  $new_status != 'publish') ) {
		tlo_cf_purge_blog_cron();
	}
}
add_action(  'transition_post_status',  'on_all_status_transitions', 10, 3 );

add_action( 'transition_comment_status', 'tlo_cf_purge_comment_status', 91, 3 );
add_action( 'edit_comment', 'tlo_cf_purge_comment_edit' );
add_action( 'pre_comment_approved', 'tlo_cf_purge_comment_new', 91, 2 );
add_action( 'trashed_post', 'tlo_cf_purge_blog_cron', 10, 0);
add_action( 'switch_theme', 'tlo_cf_purge_blog_cron', 10, 0);
add_action( '_core_updated_successfully', 'tlo_cf_purge_blog_cron', 10, 0);
add_action( 'init', 'tlo_cf_register_publish_hooks', 99 );

function tlo_cf_add_links( $links, $file ) {
	if ( $file == plugin_basename( __FILE__ ) ) {
		$posk_links = '<a href="https://wordpress.org/plugins/full-site-cache-cf/installation/">'.__('See Installation Page', 'full-site-cache-cf').'</a>';
		// Make sure the 'Settings' link at first
		array_unshift( $links, $posk_links );
	}
	return $links;
}
add_filter( 'plugin_action_links', 'tlo_cf_add_links', 91, 2 );


/**
* Invalidate cache with XML file
*
* @since   0.0.1
* @change  0.0.1
*
* @param   array  $xml
*/
function tlo_cf_purge($xml){
	global $tlo_cf_id;
	$distribution = $tlo_cf_id[get_current_blog_id()];
	$access_key = AWS_ACCESS_KEY_ID;
	$secret_key = AWS_SECRET_ACCESS_KEY;

	$len = strlen($xml);
	$date = gmdate('D, d M Y G:i:s T');
	$sig = base64_encode(
		hash_hmac('sha1', $date, $secret_key, true)
	);

	$msg = "POST /2010-11-01/distribution/{$distribution}/invalidation HTTP/1.0\r\n";
	$msg .= "Host: cloudfront.amazonaws.com\r\n";
	$msg .= "Date: {$date}\r\n";
	$msg .= "Content-Type: text/xml; charset=UTF-8\r\n";
	$msg .= "Authorization: AWS {$access_key}:{$sig}\r\n";
	$msg .= "Content-Length: {$len}\r\n\r\n";
	$msg .= $xml;

	$fp = fsockopen('ssl://cloudfront.amazonaws.com', 443, 
		$errno, $errstr, 30
	);
	if (!$fp) {
		return;
	}
	fwrite($fp, $msg);
	$resp = '';
	while(! feof($fp)) {
		$resp .= fgets($fp, 1024);
	}
	fclose($fp);
	return $resp;
}